@@ -1,5 +1,5 @@
 package Authen::NZRealMe;
-$Authen::NZRealMe::VERSION = '1.20';
+$Authen::NZRealMe::VERSION = '1.21';
 use warnings;
 use strict;
 
@@ -163,7 +163,7 @@ sub _dispatch_make_req {
         my $allow_create = $opt->{allow_create} ? 1 : 0;
         push @req_options, allow_create => $allow_create;
     }
-    my $acs_index = $opt->{acs_index} // 0;
+    my $acs_index = $opt->{acs_index} // 'default';
     push @req_options, acs_index => $acs_index;
 
     my $req = $sp->new_request( @req_options );
